unit CustomMarshallerTestU;

interface

uses
  TestFramework,
  CustomConverter,
  MyObjects;

type
  TCustomMarshallerTest = class(TTestCase)
  published
    procedure TestSimpleObject;
    procedure TestSimpleObjectWithISODate;
    procedure TestComplexObject;
    procedure TestComplexObjectWithNestedObject;
  end;

implementation

uses
  DBXJSON,
  DBXJSONReflect,
  Classes, SysUtils;

{ TCustomMarshallerTest }

procedure TCustomMarshallerTest.TestComplexObject;
var
  Marshaller: TJSONMarshal;
  UnMarshaller: TJSONUnMarshal;
  Teenager: TTeenager;
  Value, JSONTeenager: TJSONObject;
begin
  Marshaller := TJSONMarshal.Create(TJSONConverter.Create);
  try
    Marshaller.RegisterConverter(TTeenager, 'BornDate', ISODateTimeConverter);
    Marshaller.RegisterConverter(TStringList, StringListConverter);

    Teenager := TTeenager.CreateAndInitialize;
    try
      Value := Marshaller.Marshal(Teenager) as TJSONObject;
    finally
      Teenager.Free;
    end;


    // UnMarshalling Teenager
    UnMarshaller := TJSONUnMarshal.Create;
    try
      UnMarshaller.RegisterReverter(TTeenager, 'BornDate', ISODateTimeReverter);
      UnMarshaller.RegisterReverter(TStringList, StringListReverter);

      Teenager := UnMarshaller.Unmarshal(Value) as TTeenager;
      try
        CheckEquals('Daniele', Teenager.FirstName);
        CheckEquals('Teti', Teenager.LastName);
        CheckEquals(29, Teenager.Age);
        CheckEquals(EncodeDate(1979, 11, 4), Teenager.BornDate);
        CheckEquals(3, Teenager.Phones.Count);
        CheckEquals('NUMBER01', Teenager.Phones[0]);
        CheckEquals('NUMBER02', Teenager.Phones[1]);
        CheckEquals('NUMBER03', Teenager.Phones[2]);
      finally
        Teenager.Free;
      end;
    finally
      UnMarshaller.Free;
    end;
  finally
    Marshaller.Free;
  end;

end;

procedure TCustomMarshallerTest.TestComplexObjectWithNestedObject;
var
  Marshaller: TJSONMarshal;
  UnMarshaller: TJSONUnMarshal;
  Programmer: TProgrammer;
  Value, JSONProgrammer: TJSONObject;
begin
  Marshaller := TJSONMarshal.Create(TJSONConverter.Create);
  try
    Marshaller.RegisterConverter(TProgrammer, 'BornDate', ISODateTimeConverter);
    Marshaller.RegisterConverter(TStringList, StringListConverter);
    Marshaller.RegisterConverter(TProgrammer, 'Laptops', LaptopListConverter);
    Programmer := TProgrammer.CreateAndInitialize;
    try
      Value := Marshaller.Marshal(Programmer) as TJSONObject;
    finally
      Programmer.Free;
    end;

    // UnMarshalling Programmer
    UnMarshaller := TJSONUnMarshal.Create;
    try
      UnMarshaller.RegisterReverter(TProgrammer, 'BornDate', ISODateTimeReverter);
      UnMarshaller.RegisterReverter(TStringList, StringListReverter);
      UnMarshaller.RegisterReverter(TProgrammer, 'Laptops', LaptopListReverter);

      Programmer := UnMarshaller.Unmarshal(Value) as TProgrammer;
      try
        CheckEquals('Daniele', Programmer.FirstName);
        CheckEquals('Teti', Programmer.LastName);
        CheckEquals(29, Programmer.Age);
        CheckEquals(EncodeDate(1979, 11, 4), Programmer.BornDate);
        CheckEquals(3, Programmer.Phones.Count);
        CheckEquals('NUMBER01', Programmer.Phones[0]);
        CheckEquals('NUMBER02', Programmer.Phones[1]);
        CheckEquals('NUMBER03', Programmer.Phones[2]);
        CheckEquals('HP Presario C700',Programmer.Laptops[0].Model);
        CheckEquals(1000,Programmer.Laptops[0].Price);
        CheckEquals('Toshiba Satellite Pro',Programmer.Laptops[1].Model);
        CheckEquals(800,Programmer.Laptops[1].Price);
        CheckEquals('IBM Travelmate 500',Programmer.Laptops[2].Model);
        CheckEquals(1300,Programmer.Laptops[2].Price);
      finally
        Programmer.Free;
      end;
    finally
      UnMarshaller.Free;
    end;
  finally
    Marshaller.Free;
  end;

end;

procedure TCustomMarshallerTest.TestSimpleObject;
var
  Marshaller: TJSONMarshal;
  UnMarshaller: TJSONUnMarshal;
  Kid: TKid;
  Value, JSONKid: TJSONObject;
begin
  Marshaller := TJSONMarshal.Create(TJSONConverter.Create);
  try
    Kid := TKid.CreateAndInitialize;
    try
      Value := Marshaller.Marshal(Kid) as TJSONObject;

      // Full qualified class name
      CheckEquals('MyObjects.TKid', Value.Get(0).JsonValue.Value);

      // Object ID
      CheckEquals('1', Value.Get(1).JsonValue.Value);

      // Reference to the object fields
      JSONKid := Value.Get(2).JsonValue as TJSONObject;
      CheckEquals('FirstName', JSONKid.Get(0).JsonString.Value);
      CheckEquals('Daniele', JSONKid.Get(0).JsonValue.Value);

      CheckEquals('LastName', JSONKid.Get(1).JsonString.Value);
      CheckEquals('Teti', JSONKid.Get(1).JsonValue.Value);

      CheckEquals('Age', JSONKid.Get(2).JsonString.Value);
      CheckEquals('29', JSONKid.Get(2).JsonValue.Value);

      CheckEquals('BornDate', JSONKid.Get(3).JsonString.Value);
      CheckEquals(FloatToStr(EncodeDate(1979, 11, 4)), JSONKid.Get(3).JsonValue.Value);

    finally
      Kid.Free;
    end;

    // UnMarshalling Kid
    UnMarshaller := TJSONUnMarshal.Create;
    try
      Kid := UnMarshaller.Unmarshal(Value) as TKid;
      try
        CheckEquals('Daniele', Kid.FirstName);
        CheckEquals('Teti', Kid.LastName);
        CheckEquals(29, Kid.Age);
        CheckEquals(EncodeDate(1979, 11, 4), Kid.BornDate);
      finally
        Kid.Free;
      end;
    finally
      UnMarshaller.Free;
    end;
  finally
    Marshaller.Free;
  end;
end;

procedure TCustomMarshallerTest.TestSimpleObjectWithISODate;
var
  Marshaller: TJSONMarshal;
  UnMarshaller: TJSONUnMarshal;
  Kid: TKid;
  Value, JSONKid: TJSONObject;
begin
  Marshaller := TJSONMarshal.Create(TJSONConverter.Create);
  try
    Marshaller.RegisterConverter(TKid, 'BornDate', ISODateTimeConverter);
    Kid := TKid.CreateAndInitialize;
    try
      Value := Marshaller.Marshal(Kid) as TJSONObject;

      // Full qualified class name
      CheckEquals('MyObjects.TKid', Value.Get(0).JsonValue.Value);

      // Object ID
      CheckEquals('1', Value.Get(1).JsonValue.Value);

      // Reference to the object fields
      JSONKid := Value.Get(2).JsonValue as TJSONObject;
      CheckEquals('FirstName', JSONKid.Get(0).JsonString.Value);
      CheckEquals('Daniele', JSONKid.Get(0).JsonValue.Value);

      CheckEquals('LastName', JSONKid.Get(1).JsonString.Value);
      CheckEquals('Teti', JSONKid.Get(1).JsonValue.Value);

      CheckEquals('Age', JSONKid.Get(2).JsonString.Value);
      CheckEquals('29', JSONKid.Get(2).JsonValue.Value);

      CheckEquals('BornDate', JSONKid.Get(3).JsonString.Value);
      CheckEquals('1979-11-04 00:00:00', JSONKid.Get(3).JsonValue.Value);

    finally
      Kid.Free;
    end;

    // UnMarshalling Kid
    UnMarshaller := TJSONUnMarshal.Create;
    try
      UnMarshaller.RegisterReverter(TKid, 'BornDate', ISODateTimeReverter);

      Kid := UnMarshaller.Unmarshal(Value) as TKid;
      try
        CheckEquals('Daniele', Kid.FirstName);
        CheckEquals('Teti', Kid.LastName);
        CheckEquals(29, Kid.Age);
        CheckEquals(EncodeDate(1979, 11, 4), Kid.BornDate);
      finally
        Kid.Free;
      end;
    finally
      UnMarshaller.Free;
    end;
  finally
    Marshaller.Free;
  end;
end;

initialization

RegisterTest(TCustomMarshallerTest.Suite);

end.
